"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var _ = require("lodash");
var DataTable_1 = require("./DataTable");
var Paginator_1 = require("./Paginator");
var TableController = (function () {
    function TableController() {
        this.placeholder = 'search term';
        this.searchValue = "";
        this.rowsOnPageSet = [20, 30, 50];
        this.minRowsOnPage = 0;
    }
    TableController.prototype.pages = function () {
        var result = [];
        var totalPages = this.paginator.lastPage;
        this.addPage(1, totalPages, result);
        this.addPage((this.paginator.activePage + 1) / 2, totalPages, result);
        this.addPage(this.paginator.activePage - 1, totalPages, result);
        this.addPage((this.paginator.activePage + totalPages) / 2, totalPages, result);
        this.addPage(this.paginator.activePage, totalPages, result);
        this.addPage(this.paginator.activePage + 1, totalPages, result);
        this.addPage(totalPages, totalPages, result);
        if (result.length < 7) {
            this.addPage((this.paginator.activePage + totalPages / 10), totalPages, result);
            this.addPage(this.paginator.activePage - totalPages / 10, totalPages, result);
        }
        result.sort(function (a, b) { return a - b; });
        return result;
    };
    TableController.prototype.addPage = function (page, totalPages, pages) {
        var p = Math.round(page);
        if (p > 0 && p <= totalPages && pages.indexOf(p) < 0) {
            pages.push(p);
        }
    };
    TableController.prototype.search = function (event) {
        var searchTerm = event.srcElement.value.toLowerCase();
        var index = 0;
        for (var _i = 0, _a = this.getSortedData(); _i < _a.length; _i++) {
            var row = _a[_i];
            var v = row[this.searchColumn].toString();
            if (v.toLowerCase().startsWith(searchTerm)) {
                var p = Math.floor(index / this.paginator.rowsOnPage);
                this.paginator.setPage(p + 1);
                break;
            }
            index++;
        }
    };
    TableController.prototype.getSortedData = function () {
        var data = this.table.inputData;
        var sortBy = this.table.sortBy;
        var sortOrder = this.table.sortOrder === "asc" ? "asc" : "desc";
        if (typeof sortBy === 'string' || sortBy instanceof String) {
            data = _.orderBy(data, this.caseInsensitiveIteratee(sortBy), [sortOrder]);
        }
        else {
            data = _.orderBy(data, sortBy, [sortOrder]);
        }
        return data;
    };
    TableController.prototype.caseInsensitiveIteratee = function (sortBy) {
        return function (row) {
            var value = row;
            for (var _i = 0, _a = sortBy.split('.'); _i < _a.length; _i++) {
                var sortByProperty = _a[_i];
                if (value) {
                    value = value[sortByProperty];
                }
            }
            if (value && typeof value === 'string' || value instanceof String) {
                return value.toLowerCase();
            }
            return value;
        };
    };
    __decorate([
        core_1.Input('table'),
        __metadata("design:type", DataTable_1.DataTable)
    ], TableController.prototype, "table", void 0);
    __decorate([
        core_1.Input('searchColumn'),
        __metadata("design:type", String)
    ], TableController.prototype, "searchColumn", void 0);
    __decorate([
        core_1.Input('placeholder'),
        __metadata("design:type", String)
    ], TableController.prototype, "placeholder", void 0);
    __decorate([
        core_1.ViewChild(Paginator_1.Paginator),
        __metadata("design:type", Paginator_1.Paginator)
    ], TableController.prototype, "paginator", void 0);
    TableController = __decorate([
        core_1.Component({
            selector: 'mfTableController',
            template: "\n    <div class=\"extstats-table-controller\">\n      <mfPaginator #p [mfTable]=\"table\">\n        <div class=\"extstats-pagination\" *ngIf=\"p.dataLength > p.rowsOnPage\">\n          <div class=\"help\">Page:</div>\n          <button type=\"button\" class=\"extstats-page-item\" *ngFor=\"let page of pages()\" [class.active]=\"p.activePage === page\" (click)=\"p.setPage(page)\">\n            <a class=\"extstats-page-link\" style=\"cursor: pointer\">{{page}}</a>\n          </button>\n        </div>\n\n        <div class=\"filler\"></div>\n\n        <div *ngIf=\"!!searchColumn\">\n          <div class=\"help\">Search:</div>\n          <input type=\"text\" [value]=\"searchValue\" [placeholder]=\"placeholder\" (input)=\"search($event)\">\n        </div>\n\n        <div class=\"filler\" *ngIf=\"!!searchColumn\"></div>\n\n        <div class=\"extstats-pagination\" *ngIf=\"p.dataLength > minRowsOnPage\">\n          <div class=\"help\">Rows per page:</div>\n          <button type=\"button\" class=\"extstats-page-item\" *ngFor=\"let rows of rowsOnPageSet\" [class.active]=\"p.rowsOnPage===rows\" (click)=\"p.setRowsOnPage(rows)\">\n            <a class=\"extstats-page-link\" style=\"cursor: pointer\">{{rows}}</a>\n          </button>\n        </div>\n      </mfPaginator>\n    </div>"
        })
    ], TableController);
    return TableController;
}());
exports.TableController = TableController;
//# sourceMappingURL=TableController.js.map